/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsUtilityString.c
    
Abstract:

    Implementation of sting related functions

--*/

#include <stdio.h>

#include "EmsUtilityString.h"

UINT32
AsciiToUint32 (
  IN INT8 Ch
  )
/*++

Routine Description:

  Convert character to its integer value

Arguments:

  Ch  - a character

Returns:

  its integer value or -1, if error

--*/
{
  if (Ch >= '0' && Ch <= '9') {
    return Ch - '0';
  }

  if (Ch >= 'a' && Ch <= 'f') {
    return Ch - 'a' + 10;
  }

  if (Ch >= 'A' && Ch <= 'F') {
    return Ch - 'A' + 10;
  }

  return -1;
}

UINT32
strcmp_i (
  IN CONST INT8 *S1,
  IN CONST INT8 *S2
  )
/*++

Routine Description:

  Case insensitive string compare routine.

Arguments:

  S1  - string 1
  S2  - string 2

Returns:

  0, if same

--*/
{
  while (Upper (*S1) == Upper (*S2)) {
    S2++;
    if (*S1++ == 0) {
      return 0;
    }
  }

  return * (CONST UINT8 *) S1 -*(CONST UINT8 *) S2;
}

UINT32
strncmp_i (
  IN CONST INT8   *S1,
  IN CONST INT8   *S2,
  IN UINT32       Len
  )
/*++

Routine Description:

  Case insensitive string compare routine.

Arguments:

  S1  - string 1
  S2  - string 2
  Len - length to compare

Returns:

  0, if same

--*/
{
  while (Len && (Upper (*S1) == Upper (*S2))) {
    S2++;
    if (*S1++ == 0) {
      return 0;
    }

    Len--;
  }

  return Len ? (*(CONST UINT8 *) S1 -*(CONST UINT8 *) S2) : 0;
}

INT32
AsciiStringToUint32 (
  IN CONST INT8    *Str,
  IN OUT UINT32    *Value
  )
/*++

Routine Description:

  Convert a sting to integer value.

Arguments:

  Str   - string to convert
  Value - the converse result

Returns:

  the count of the bytes converted in the string.

--*/
{
  UINT32  Index;
  UINT32  Temp;
  BOOLEAN negative;

  Index = 0;
  Temp  = 0;

  if (Str[0] == '-') {
    negative  = TRUE;
    Index     = 1;
  } else {
    negative = FALSE;
  }

  if (Str[Index] == '0' && (Str[Index + 1] == 'x' || Str[Index + 1] == 'X')) {
    Index += 2;
    while (IsDigit (Str[Index]) || IsHexDigit (Str[Index])) {
      Temp = Temp * 16 + AsciiToUint32 (Str[Index]);
      Index += 1;
    }

    *Value = Temp;
    return Index;
  }

  while (IsDigit (Str[Index])) {
    Temp = Temp * 10 + AsciiToUint32 (Str[Index]);
    Index += 1;
  }

  if (negative) {
    Temp = (UINT32) (-Temp);
  }

  *Value = Temp;
  return Index;
}

INT32
AsciiStringToUint64 (
  IN CONST INT8   *Str,
  IN OUT UINT64   *Value
  )
/*++

Routine Description:

  Convert a sting to integer value.

Arguments:

  Str   - string to convert
  Value - the converse result

Returns:

  the count of the bytes converted in the string.

--*/
{
  UINT32  Index;
  UINT64  Temp;

  Index = 0;
  Temp  = 0;

  if (Str[Index] == '0' && (Str[Index + 1] == 'x' || Str[Index + 1] == 'X')) {
    Index += 2;
    while (IsDigit (Str[Index]) || IsHexDigit (Str[Index])) {
      Temp = Temp * 16 + AsciiToUint32 (Str[Index]);
      Index += 1;
    }

    *Value = Temp;
    return Index;
  }

  while (IsDigit (Str[Index])) {
    Temp = Temp * 10 + AsciiToUint32 (Str[Index]);
    Index += 1;
  }

  *Value = Temp;
  return Index;
}

INT32
HexAsciiStringToUint32 (
  IN CONST INT8   *Str,
  IN OUT UINT32   *Value
  )
/*++

Routine Description:

  Convert a hex sting to integer value.

Arguments:

  Str   - string to convert
  Value - the converse result

Returns:

  the count of the bytes converted in the string.

--*/
{
  UINT32  Index;
  UINT32  Temp;
  Index = 0;
  Temp  = 0;

  while (IsDigit (Str[Index]) || IsHexDigit (Str[Index])) {
    Temp = Temp * 16 + AsciiToUint32 (Str[Index]);
    Index += 1;
  }

  *Value = Temp;
  return Index;
}

INT32
AsciiStringToIpv4 (
  IN CONST INT8   *Str,
  IN OUT UINT32   *Value
  )
/*++

Routine Description:

  Convert a sting to IP address.

Arguments:

  Str   - string to convert
  Value - the converse result

Returns:

  the count of the bytes converted in the string.

--*/
{
  UINT8   Index;
  UINT32  Len;
  INT32   templen;
  CONST INT8    *s;
  UINT32  addr;
  UINT32  Temp;

  Len     = 0;
  s       = Str;
  addr    = 0;

  *Value  = 0;
  for (Index = 0; Index < 4; Index++) {
    templen = AsciiStringToUint32 (s, &Temp);
    if (templen <= 0 || Temp > 255) {
      goto ErrorExit;
    }

    addr = (addr << 8) + Temp;
    Len += templen;
    s = &Str[Len];
    if (Index == 3) {
      break;
    }

    if (s[0] != '.') {
      goto ErrorExit;
    }

    Len++;
    s = &Str[Len];
  }

  *Value = addr;
  return Len;
ErrorExit:
  return 0;
}

INT32
AsciiStringToMac (
  IN CONST INT8  *Str,
  IN OUT UINT8   *Value
  )
/*++

Routine Description:

  Convert a sting to MAC address.

Arguments:

  Str   - string to convert
  Value - the converse result

Returns:

  the count of the bytes converted in the string.

--*/
{
  UINT8   Index;
  UINT32  Len;
  INT32   templen;
  CONST INT8    *s;
  UINT32  Temp;

  Len     = 0;
  s       = Str;

  *Value  = 0;
  for (Index = 0; Index < 6; Index++) {
    templen = HexAsciiStringToUint32 (s, &Temp);
    if (templen <= 0 || Temp > 255) {
      goto ErrorExit;
    }

    Value[Index] = Temp;
    Len += templen;
    s = &Str[Len];
    if (Index == 5) {
      break;
    }

    if (s[0] != ':') {
      goto ErrorExit;
    }

    Len++;
    s = &Str[Len];
  }

  return Len;
ErrorExit:
  return 0;
}

INT32
Ipv4ToAsciiString (
  IN OUT INT8   *Str,
  IN     UINT32 ip
  )
/*++

Routine Description:

  Convert IP address to a string.

Arguments:

  Str - string to convert
  ip  - the converse result

Returns:

  the count of the bytes converted in the string.

--*/
{
  INT32 Len;

  Len = sprintf (
          Str,
          "%d.%d.%d.%d",
          ip >> 24,
          ip >> 16 & 0xFF,
          ip >> 8 & 0xFF,
          ip & 0xFF
          );
  return Len;
}

INT32
MacToAsciiString (
  IN OUT INT8    *Str,
  IN CONST UINT8 *Mac
  )
/*++

Routine Description:

  Convert MAC address to a string.

Arguments:

  Str - string to convert
  Mac - the converse result

Returns:

  the count of the bytes converted in the string.

--*/
{
  INT32 Len;

  Len = sprintf (
          Str,
          "%02x:%02x:%02x:%02x:%02x:%02x",
          Mac[0],
          Mac[1],
          Mac[2],
          Mac[3],
          Mac[4],
          Mac[5]
          );
  return Len;
}

STATIC
INT32
GetValidString (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "GetValidString" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK or TCL_ERROR

--*/
{
  INT8  ErrorBuff[512];

  if(Argc != 2)
  {
    sprintf (ErrorBuff, "Syntax of GetAsiciiString - GetAsiciiString String\n");
    goto ErrorExit;
  }

  /*
   * what is the matter? Argv[*][*] is not char type ?
   */
  if(Argv[1][0] == 0xFFFFFFEF && 
  	Argv[1][1] == 0xFFFFFFBB && 
  	Argv[1][2] == 0xFFFFFFBF)
    Tcl_AppendResult (Interp, Argv[1] + 3, (INT8 *)NULL);
  else
  	Tcl_AppendResult (Interp, Argv[1], (INT8 *)NULL);

  return TCL_OK;
ErrorExit:
  Tcl_AppendResult (Interp, ErrorBuff, (INT8 *) NULL);
  return TCL_ERROR;
}

VOID_P
StringUtilityInit (
  IN Tcl_Interp *Interp
  )
/*++

Routine Description:

  EMS string related TCL initialization.

Arguments:

  Interp      - TCL intepreter.

Returns:

  None

--*/
{
  Tcl_CreateCommand (
    Interp,
    "GetValidString",
    GetValidString,
    (ClientData) NULL,
    (Tcl_CmdDeleteProc *) NULL
    );
}
